import { world, system, ItemStack } from "@minecraft/server";
import { ActionFormData, ModalFormData, MessageFormData } from "@minecraft/server-ui";

let buttons = []; // Array to store button objects

world.afterEvents.itemUse.subscribe((event) => {
    const item = event.itemStack;
    const player = event.source;

    if (item.typeId == "um:itemxchange") {
        loadButtons()
        howmmenu(player);
    }
});


// Save the buttons array to the dynamic property
function saveButtons() {
    // Convert the buttons array to a string and save it to the world dynamic property
    world.setDynamicProperty("sevedata", JSON.stringify(buttons));
}

// Retrieve the buttons array from the dynamic property
function loadButtons() {
    // Retrieve the string data and convert it back into an array
    const savedButtons = world.getDynamicProperty("sevedata");
    if (savedButtons) {
        buttons = JSON.parse(savedButtons); // Deserialize the data
    } else {
        buttons = []; // Initialize as empty if no data is found
    }
}


function howmmenu(player) {
  const form = new ActionFormData()
  form.title("ItemXchange")
  form.button("Add New Trade")
  form.button("View Trades")
  form.show(player).then(res =>{
    if(res.canceled) return;
    if(res.selection == 0){
      addButton(player);
    }
    if(res.selection == 1){
      
      itemXchangeUI(player);
    }
  })
}


function itemXchangeUI(player) {
    const form = new ActionFormData();
    form.title("ItemXchange");

    buttons.forEach((button, index) => {
        // Remove "minecraft:" prefix if present
        let cleanItemName = button.itemName.replace(/^minecraft:/, "");
        // Replace unwanted symbols with spaces
        cleanItemName = cleanItemName.replace(/[^a-zA-Z0-9 ]/g, " ");
        // Capitalize each word
        cleanItemName = cleanItemName
            .split(" ") // Split into words
            .map(word => word.charAt(0).toUpperCase() + word.slice(1).toLowerCase()) // Capitalize the first letter
            .join(" "); // Join the words back with a space

        form.button(`${cleanItemName} (Amount: ${button.amount})`);
    });

    form.show(player).then((res) => {
        if (res.canceled) howmmenu(player);
        if (res.selection >= 0) {
            const selectedButton = buttons[res.selection];
            const index = res.selection;
            buysell(player, selectedButton, index);
        }
    });
}

function addButton(player) {
    const inventory = player.getComponent("inventory").container; // Get player's inventory
    const itemMap = new Map(); // Use a Map to group items by type

    // Loop through inventory slots to collect item data
    for (let i = 0; i < inventory.size; i++) {
        const item = inventory.getItem(i);
        if (item) {
            const itemName = item.typeId;
            const currentAmount = itemMap.has(itemName) ? itemMap.get(itemName) : 0;
            itemMap.set(itemName, currentAmount + item.amount); // Sum up the amount of each item
        }
    }

    if (itemMap.size === 0) {
        player.sendMessage("Your inventory is empty. Add some items first!");
        return;
    }

    const add = new ModalFormData();
    add.title("Add trade");

    // Create a dropdown for the main item
    const itemNames = Array.from(itemMap.keys()).map((itemName) => {
        const totalAmount = itemMap.get(itemName);
        return `${itemName} (Amount: ${totalAmount})`;
    });
    add.dropdown("Select Item", itemNames); // Dropdown for selecting the main item
    add.textField("Enter Amount", "Enter amount"); // Text field for the main item amount

    // Create a dropdown for the trade item
    add.dropdown("Select Trade Item", itemNames); // Dropdown for selecting the trade item
    add.textField("Enter Trade Amount", "Enter trade amount"); // Text field for trade item amount

    // Add new text field for codinet
    add.textField("Enter Codinet", "00 00 00"); // Text field for codinet value

    add.show(player).then((res) => {
        if (res.canceled) howmmenu(player);

        const selectedItemIndex = res.formValues[0];
        const enteredAmount = parseInt(res.formValues[1], 10);
        const tradeItemIndex = res.formValues[2];
        const tradeAmount = parseInt(res.formValues[3], 10);
        const codinet = res.formValues[4]; // Get the codinet value entered by the player

        const selectedItemName = Array.from(itemMap.keys())[selectedItemIndex];
        const selectedItemAmount = itemMap.get(selectedItemName);

        const tradeItemName = Array.from(itemMap.keys())[tradeItemIndex];
        const tradeItemAmount = itemMap.get(tradeItemName);

        // Validate entered amount
        if (isNaN(enteredAmount) || enteredAmount <= 0 || enteredAmount > selectedItemAmount) {
            player.sendMessage(
                `Error: Entered amount (${enteredAmount}) exceeds available amount (${selectedItemAmount}) for ${selectedItemName}.`
            );
            return;
        }

        // Validate trade amount
        if (isNaN(tradeAmount) || tradeAmount <= 0) {
            player.sendMessage("Error: Trade amount must be a positive number.");
            return;
        }

        // Validate codinet input
        if (!codinet) {
            player.sendMessage("Error: Codinet cannot be empty.");
            return;
        }

        // Remove the selected item and amount from the player's inventory
        let remainingToRemove = enteredAmount;

        for (let i = 0; i < inventory.size && remainingToRemove > 0; i++) {
            const item = inventory.getItem(i);

            if (item && item.typeId === selectedItemName) {
                if (item.amount <= remainingToRemove) {
                    // Fully consume the stack
                    remainingToRemove -= item.amount;
                    inventory.setItem(i, null); // Remove the stack from inventory
                } else {
                    // Partially consume the stack
                    item.amount -= remainingToRemove;
                    inventory.setItem(i, item);
                    remainingToRemove = 0; // All required items are deducted
                }
            }
        }

        // If still remainingToRemove > 0, that means the inventory didn't have enough items
        if (remainingToRemove > 0) {
            player.sendMessage(`Error: Not enough items in your inventory to complete the trade.`);
            return;
        }

        // Add button to the list
        buttons.push({
            itemName: selectedItemName,
            amount: enteredAmount,
            tradeItem: tradeItemName,
            tradeAmount,
            addedBy: player.name,
            codinet, // Push codinet to the array
        });

        player.sendMessage("Button added successfully!");
        saveButtons();
    });
}
function buysell(player, button, index) {
    const buy = new MessageFormData();

    buy.title("Buy");
    buy.body(
        `§6Buy Item Details:§r\n` +
        `§bItem Name: §e${button.itemName}§r\n` +
        `§bItem Amount: §e${button.amount}§r\n` +
        `§bTrade Item Name: §e${button.tradeItem}§r\n` +
        `§bTrade Amount: §e${button.tradeAmount}§r\n` +
        `\n§7Added by: §9${button.addedBy || "Unknown"}§r`
    );

    buy.button1("§aYes"); // Green for "Yes"
    buy.button2("§cNo");  // Red for "No"

    buy.show(player).then((res) => {
        if (res.canceled || res.selection === 1) itemXchangeUI(player);

        if (res.selection === 0) {
            const inventory = player.getComponent("inventory").container;

            // Check if the player has the trade item and enough amount
            let tradeSlot = null;
            let totalTradeAmount = 0; // Track the total amount of the trade item

            // Loop through inventory and calculate total trade item amount
            for (let i = 0; i < inventory.size; i++) {
                const item = inventory.getItem(i);
                if (item && item.typeId === button.tradeItem) {
                    totalTradeAmount += item.amount;
                    if (totalTradeAmount >= button.tradeAmount) {
                        tradeSlot = i; // Found enough of the trade item
                        break;
                    }
                }
            }

            if (totalTradeAmount < button.tradeAmount) {
                player.sendMessage(
                    `§cTrade failed! You don't have enough ${button.tradeItem} (Required: ${button.tradeAmount}).`
                );
                return;
            }

            // Deduct the trade item from the player's inventory
            let remainingAmount = button.tradeAmount;
            for (let i = 0; i < inventory.size && remainingAmount > 0; i++) {
                const item = inventory.getItem(i);
                if (item && item.typeId === button.tradeItem) {
                    if (item.amount <= remainingAmount) {
                        remainingAmount -= item.amount;
                        inventory.setItem(i, null); // Remove the item if fully used
                    } else {
                        item.amount -= remainingAmount;
                        inventory.setItem(i, item); // Update the item with the new amount
                        remainingAmount = 0; // All required amount is deducted
                    }
                }
            }

            // Add the purchased item to the player's inventory
            const purchasedItem = new ItemStack(button.itemName, button.amount);
            inventory.addItem(purchasedItem);

            player.sendMessage(
                `§aTrade successful! You received ${button.amount}x ${button.itemName}.\n` +
                `§7This trade was set up by: §9${button.addedBy || "Unknown"}§r`
            );

            // Add trade item to chest using codinet coordinates
            const codinetArray = button.codinet.split(" ");
            const x = parseInt(codinetArray[0], 10);
            const y = parseInt(codinetArray[1], 10);
            const z = parseInt(codinetArray[2], 10);
            addTradeItemToChest(button.tradeItem, button.tradeAmount, x, y, z); // Add item to chest

            // Remove the button after successful trade
            buttons.splice(index, 1);
            saveButtons();
        }
    });
}

function addTradeItemToChest(tradeItem, tradeAmount, x, y, z) {
    const chestLocation = { x: x, y: y, z: z }; // Use the parsed coordinates

    const chestBlock = world.getDimension("overworld").getBlock(chestLocation); // Get the block at the chest location
    if (chestBlock && chestBlock.typeId === "minecraft:chest") {
        const chestContainer = chestBlock.getComponent("inventory").container;

        // Add the trade item to the chest's inventory
        const itemStack = new ItemStack(tradeItem, tradeAmount);
        chestContainer.addItem(itemStack);

        // Send confirmation message
        world.getPlayers().forEach((player) => {
            player.sendMessage(
                `§aThe trade item ${tradeAmount}x ${tradeItem} has been added to the chest at the trade location.`
            );
        });
    } else {
        console.log("Error: No chest found at the specified location.");
    }
}